from playwright.async_api import Page as PageAsync
from playwright.sync_api import Page as PageSync
from selenium.webdriver.remote.webdriver import WebDriver

from ._base import BrowserAdapter
from .playwright import PlaywrightAsyncAdapter, PlaywrightSyncAdapter
from .selenium import SeleniumAdapter
from .types import AnyDriver

# TODO: make selenium and playwright drivers optional


def adapter_factory(driver: AnyDriver) -> BrowserAdapter:
    if isinstance(driver, WebDriver):
        return SeleniumAdapter(driver)
    elif isinstance(driver, PageSync):
        return PlaywrightSyncAdapter(driver)
    elif isinstance(driver, PageAsync):
        return PlaywrightAsyncAdapter(driver)
    # TODO: add support for Puppeteer

    else:
        raise ValueError(
            "Invalid driver type: please provide a Selenium WebDriver or a Playwright Page"
        )


__all__ = [
    "BrowserAdapter",
    "PlaywrightAsyncAdapter",
    "PlaywrightSyncAdapter",
    "SeleniumAdapter",
    "adapter_factory",
    "AnyDriver",
]



from typing import TypedDict, Union

from playwright.async_api import Page as PageAsync
from playwright.sync_api import Page as PageSync
from selenium.webdriver.remote.webdriver import WebDriver

AnyDriver = Union[WebDriver, PageSync, PageAsync]


class ViewPortSize(TypedDict):
    width: int
    height: int
    content_height: int



from pathlib import Path
from typing import Dict, Tuple

from tarsier._base import ITarsier
from tarsier._utils import load_js
from tarsier.adapter import AnyDriver, BrowserAdapter, adapter_factory
from tarsier.ocr import OCRService

TagToXPath = Dict[int, str]


class Tarsier(ITarsier):
    _JS_TAG_UTILS = Path(__file__).parent / "tag_utils.min.js"

    def __init__(self, ocr_service: OCRService):
        self._ocr_service = ocr_service
        self._js_utils = load_js(self._JS_TAG_UTILS)

    async def page_to_image(
        self, driver: AnyDriver, tag_text_elements: bool = False
    ) -> Tuple[bytes, Dict[int, str]]:
        adapter = adapter_factory(driver)
        tag_to_xpath = await self._tag_page(adapter, tag_text_elements)
        screenshot = await self._take_screenshot(adapter)
        await self._remove_tags(adapter)
        return screenshot, tag_to_xpath

    async def page_to_text(
        self, driver: AnyDriver, tag_text_elements: bool = False
    ) -> Tuple[str, TagToXPath]:
        image, tag_to_xpath = await self.page_to_image(driver, tag_text_elements)
        page_text = self._run_ocr(image)
        return page_text, tag_to_xpath

    @staticmethod
    async def _take_screenshot(adapter: BrowserAdapter) -> bytes:
        viewport = await adapter.get_viewport_size()
        default_width = viewport["width"]

        await adapter.set_viewport_size(default_width, viewport["content_height"])
        screenshot = await adapter.take_screenshot()
        await adapter.set_viewport_size(default_width, viewport["height"])

        return screenshot

    def _run_ocr(self, image: bytes) -> str:
        ocr_text = self._ocr_service.annotate(image)
        page_text = self._ocr_service.format_text(ocr_text)
        return page_text

    async def _tag_page(
        self, adapter: BrowserAdapter, tag_text_elements: bool = False
    ) -> Dict[int, str]:
        await adapter.run_js(self._js_utils)

        script = f"return window.tagifyWebpage({str(tag_text_elements).lower()});"
        tag_to_xpath = await adapter.run_js(script)

        return {int(key): value for key, value in tag_to_xpath.items()}

    async def _remove_tags(self, adapter: BrowserAdapter) -> None:
        script = "return window.removeTags();"

        await adapter.run_js(script)



import argparse
import asyncio
import json
from pprint import pprint

from playwright.async_api import async_playwright

from tarsier.core import Tarsier
from tarsier.ocr import GoogleVisionOCRService


async def main(credentials_path: str, url: str, verbose: bool) -> None:
    with open(credentials_path, "r") as f:
        credentials = json.load(f)

    ocr_service = GoogleVisionOCRService(credentials)
    tarsier = Tarsier(ocr_service)

    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=False)
        page = await browser.new_page()
        await page.goto(url)

        print("👀  Navigated to page.")
        print("🤖  Running Tarsier OCR...")

        page_text, tag_to_xpath = await tarsier.page_to_text(page)

        if verbose:
            print("XPaths:")
            pprint(tag_to_xpath)

        print("Page Text:")
        print(page_text)
        print("✅  Successfully OCR'd page text.")


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description="Tarsier OCR command line interface.")
    parser.add_argument(
        "credentials_path", help="Path to the Google credentials JSON file", type=str
    )
    parser.add_argument("url", help="URL to navigate to", type=str)
    parser.add_argument(
        "-v",
        "--verbose",
        help="Show verbose output (including xpaths)",
        action="store_true",
    )
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_args()
    asyncio.run(main(args.credentials_path, args.url, args.verbose))



from typing import List, Tuple, TypedDict


class ImageAnnotation(TypedDict):
    text: str  # the word
    midpoint: Tuple[float, float]  # the UNNORMALIZED midpoint of the word, (X,Y)
    midpoint_normalized: Tuple[
        float, float
    ]  # the normalized midpoint between 0 - 1  (X,Y)


class ImageAnnotatorResponse(TypedDict):
    words: List[ImageAnnotation]  # a list of words and their midpoints
